// Creative Commons Attribution-Share Alike 3.0 Unported Licence
// http://creativecommons.org/licenses/by-sa/3.0/

var CGD = window.CGD || {};

CGD.DRAW = CGD.DRAW || {};
// the common 'boilerplate' code for using a canvas.
CGD.DRAW.on = function(id, f, options) {
  var canvas = document.getElementById(id);
  if (!canvas) {
    return;
  }
  var context = canvas.getContext("2d");
  if (!context) {
    return;
  }

  context.save();
  context.scale(canvas.width, canvas.height);
  context.clearRect(0, 0, 1, 1);
  if (options && 'origin' in options && options.origin == 'center') {
    context.translate(0.5, 0.5);
  }

  f(context);

  context.restore();
};

// Right now about all we do is color interpolation and support code.
CGD.RGB = CGD.RGB || {};
// Some of the methods are string operations, so they go there.
CGD.STRING = CGD.STRING || {};
(function() {
  // color names are convient, but now we have to deal with them.
  CGD.RGB.STRINGS = {
    red: '#F00',
    green: '#080', // and they are sometimes surprising.
    blue: '#00F',
    orange: '#FFA500',
    yellow: '#FF0',
    purple: '#808',
    pink: '#FFC0CB',
    cyan: '#0FF',
    white: '#FFF',
    black: '#000',
    grey: '#888',
    gray: '#888',
    lastEntry: null
  };
  
  function rgbFromHex6(s) {
    function c(s, pos) {
      return parseInt(s.substr(pos,2), 16) / 255;
    }
    return {r: c(s, 1), g: c(s, 3), b: c(s, 5)};
  }

  function rgbFromHex3(s) {
    function c(s, pos) {
      return parseInt(s.substr(pos,1), 16) / 15;
    }
    return {r: c(s, 1), g: c(s, 2), b: c(s, 3)};
  }

  function rgbFromString(s) {
    if (!s) {
      return rgbFromString('white');
    } else if (CGD.RGB.STRINGS[s]) {
      return rgbFromString(CGD.RGB.STRINGS[s]);
    } else if (s.length == 4) {
      return rgbFromHex3(s);
    } else if (s.length == 7) {
      return rgbFromHex6(s);
    } else {
      throw s + ' is not a color';
    }
  }
  CGD.RGB.fromString = rgbFromString;

  function stringFromRgb(rgb) {
    function c(a) {
      var b = Math.round(a * 255).toString(16);
      return (b.length == 1) ? ('0' + b) : b;
    }
    return '#' + c(rgb.r) + c(rgb.g) + c(rgb.b);
  }
  CGD.STRING.fromRgb = stringFromRgb;

  function interpolate(a, b, by) {
    return a + (b-a)*by;
  }

  function blendColors(a, b, by) {
    return {
      r: interpolate(a.r, b.r, by),
      g: interpolate(a.g, b.g, by),
      b: interpolate(a.b, b.b, by)
    };
  }
  
  function interpolateStringColors(a, b, steps) {
    var as = rgbFromString(a);
    var bs = rgbFromString(b);
    // Pass a null through (sometimes used to draw nothing for transparency)
    //   Otherwise remove color names in case the display system 
    //   doesn't support them. (IE/excanvas doesn't do pink)
    var it = [!a ? a : stringFromRgb(as)];
    for (var i = 1;i < steps-1;i++) {
      it.push(stringFromRgb(blendColors(as, bs, i / (steps-1))));
    }
    it.push(!b ? b : stringFromRgb(bs));
    return it;
  }
  CGD.RGB.interpolate = interpolateStringColors;

  function blendStringColors(a, b, by) {
    return stringFromRgb(blendColors(rgbFromString(a), rgbFromString(b), by));
  }
  CGD.RGB.blend = blendStringColors;

}());

CGD.ARC = CGD.ARC || {CLOCKWISE: 0, COUNTERCLOCKWISE: true};
